/***************************************************************************************
 * @file     va108xx_debug.h
 * @version  V0.1
 * @date     12 February 2019
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2019 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __VA108_DEBUG_H
#define __VA108_DEBUG_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <assert.h>
#include "board.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

/** Runtime assert */
#ifdef USE_ASSERT
#define c_assert(e) assert(e)
#else
#define c_assert(e) ((void)0)
#endif
  
/** Compile time assert */
#ifdef USE_ASSERT
#define COMPILE_TIME_ASSERT(pred) \
  switch(0){case 0:case pred:;}
#else
#define COMPILE_TIME_ASSERT(pred) ((void)0)
#endif
  
/** Debug print */
#ifdef DEBUG_PRINTS
#define dbgprintf(...) DBG_Printf(__VA_ARGS__)
#define dbgprintln(...) DBG_Println(__VA_ARGS__)
#else
#define dbgprintf(...) ((void)0)
#define dbgprintln(...) ((void)0)
#endif

/** UART0/1 defs */
#ifndef VOR_UART0
#define VOR_UART0 (VOR_UARTA)
#endif

#ifndef VOR_UART1
#define VOR_UART1 (VOR_UARTB)
#endif

/*****************************************************************************/
/* Global type definitions ('typedef')                                       */ 
/*****************************************************************************/
  
typedef enum{
  en_stdio_none,
  en_stdio_uart0,
  en_stdio_uart1,
  en_stdio_porta,
  en_stdio_portb
} en_stdio_t;

/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

extern void DBG_Printf(const char *fmt, ...);
extern void DBG_Println(const char *fmt, ...);
extern void DBG_SetStdioOutput(en_stdio_t io);

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif /* __VA108_DEBUG_H */
