/***************************************************************************************
 * @file     rhd5950.h
 * @version  V1.0
 * @date     31 October 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __RHD5950_H
#define __RHD5950_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "va108xx.h"
#include "board.h"
#include "va108xx_debug.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

/* The number of channels that the ADC has. */
#define RHD5950_NUM_CHANNELS    (16)

/* Expected VREF voltage */
#define RHD5950_VREF (5.0)

/* RHD5950 MCU connections (internal to the SIP). */
#define RHD5950_STCNV_PORT   PORTB
#define RHD5950_STCNV_PIN    (4)

#define RHD5950_OE_PORT      PORTB
#define RHD5950_OE_PIN       (5)

#define RHD5950_BUSY_L_PORT  PORTB
#define RHD5950_BUSY_L_PIN   (9)

#define RHD5950_CLK_PORT     PORTB
#define RHD5950_CLK_PIN      (6)
#define RHD5950_CLK_TIM      (VOR_TIM6)
#define RHD5950_CLK_TIM_NUM  (6)
#define RHD5950_CLK_TIM_FUNSEL (3)

#define RHD5950_EN_PORT      PORTB
#define RHD5950_EN_PIN       (7)

// Will need to flip lsb-msb at use (DATA0 is ADC's MSB, DATA13 is LSB for the SIP)
#define RHD5950_DATA0_PORT   PORTA
#define RHD5950_DATA0_POS    (0)
#define RHD5950_DATA_WIDTH   (14)

#define RHD5950_MUX0_PORT    PORTB
#define RHD5950_MUX0_BANK    VOR_GPIO->BANK[1]
#define RHD5950_MUX0_PIN     (0)
#define RHD5950_MUX0_WIDTH   (4)

#define RHD5950_EOC_PORT     PORTB
#define RHD5950_EOC_PIN      (8)

#define RHD5950_CLK_PER_CONV (20)

/*****************************************************************************/
/* Global type definitions ('typedef')                                       */ 
/*****************************************************************************/

/** unsigned voltage type, units: millivolts */
#ifndef uvolt1000_t
typedef uint32_t uvolt1000_t; 
#endif

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

/* Initializes the ADC. */
extern void RHD5950_Init(void);

/* Wakes up the ADC. */
extern void RHD5950_WakeUp(void);

/* Puts the ADC to sleep. */
extern void RHD5950_Sleep(void);

/* presets the ADC channel select pins */
extern void RHD5950_setADCch(uint32_t adcch);

//only for bkwd compat
extern  uint32_t RHD5950_setadcch_and_Sample(uint32_t adcch);

/* Performs an ADC conversion with selected ch */
extern uint32_t RHD5950_Sample(void);

/* Converts an ADC sample to a four digit voltage. */
extern uvolt1000_t RHD5950_SampleToVoltage1000(uint32_t ui32Sample);

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif // __RHD5950_H
