/***************************************************************************************
 * @file     rhd5950.c
 * @version  V1.0
 * @date     31 October 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2019 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include "rhd5950.h"

/*****************************************************************************/ 
/* Local variable definitions ('static')                                     */ 
/*****************************************************************************/

/* The frequency that the SCLK pin is driven at. */
static const uint32_t m_ui32ClockHz = 1000000; // 1MHz

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

/*******************************************************************************
 **
 ** @brief  Initalizes the RHD5950 ADC.
 **
 ******************************************************************************/
void RHD5950_Init(void)
{
    /* Verify SCLK pin can be configured as a Timer output.  Check only supports
     * PORTB pins.*/
//  ASSERT(RHD5950_CLK_PORT == &(VOR_GPIO->BANK[1]));
//  ASSERT((RHD5950_CLK_PIN <= 6) || (RHD5950_CLK_PIN >= 10));
//  ASSERT(RHD5950_CLK_TIM == &(VOR_TIM->BANK[RHD5950_CLK_PIN]));
  
  uint32_t i;
    
  /* Configure pins. */
  VOR_IOCONFIG->RHD5950_CLK_PORT[RHD5950_CLK_PIN] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos);
  RHD5950_EN_PORT->CLROUT = (1UL << RHD5950_EN_PIN);
  RHD5950_EN_PORT->DIR |= (1UL << RHD5950_EN_PIN);

  VOR_IOCONFIG->RHD5950_OE_PORT[RHD5950_OE_PIN] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos);
  RHD5950_OE_PORT->SETOUT = (1UL << RHD5950_OE_PIN);
  RHD5950_OE_PORT->DIR |= (1UL << RHD5950_OE_PIN);

  VOR_IOCONFIG->RHD5950_STCNV_PORT[RHD5950_STCNV_PIN] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos);
  RHD5950_STCNV_PORT->CLROUT = (1UL << RHD5950_STCNV_PIN);
  RHD5950_STCNV_PORT->DIR |= (1UL << RHD5950_STCNV_PIN);

  for(i=0; i<RHD5950_DATA_WIDTH; i++){
    VOR_IOCONFIG->RHD5950_DATA0_PORT[RHD5950_DATA0_POS+i] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos) |
                                                                        IOCONFIG_PORTA_PEN_Msk;
  }
  RHD5950_DATA0_PORT->DIR &= ~(0x3fff << RHD5950_DATA0_POS);

  for(i=0; i<RHD5950_MUX0_WIDTH; i++){
    VOR_IOCONFIG->RHD5950_MUX0_PORT[RHD5950_MUX0_PIN+i] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos);
  }
  RHD5950_MUX0_PORT->CLROUT = (0xf << RHD5950_MUX0_PIN);
  RHD5950_MUX0_PORT->DIR |= (0xf << RHD5950_MUX0_PIN);
  
  VOR_IOCONFIG->RHD5950_EOC_PORT[RHD5950_EOC_PIN] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos) | 
                                                                IOCONFIG_PORTA_PEN_Msk;
  RHD5950_EOC_PORT->DIR &= ~(1UL << RHD5950_EOC_PIN);

  VOR_IOCONFIG->RHD5950_BUSY_L_PORT[RHD5950_BUSY_L_PIN] = (FUNSEL0 << IOCONFIG_PORTA_FUNSEL_Pos) | 
                                                                      IOCONFIG_PORTA_PEN_Msk;
  RHD5950_BUSY_L_PORT->DIR &= ~(1UL << RHD5950_BUSY_L_PIN);

  /* Configure SCLK to be driven by Timer. */
  VOR_IOCONFIG->RHD5950_CLK_PORT[RHD5950_CLK_PIN] = (VOR_IOCONFIG->RHD5950_CLK_PORT[RHD5950_CLK_PIN] &
                                       ~IOCONFIG_PORTA_FUNSEL_Msk) |
                                      ((RHD5950_CLK_TIM_FUNSEL << IOCONFIG_PORTA_FUNSEL_Pos) | 
                                       (IOCONFIG_PORTA_INVOUT_Msk) |
                                       (IOCONFIG_PORTA_IEWO_Msk));
        
  /* Configure Timer:
   *  PWM @ 1MHz, 50%, H->L
   *  Automatic restart, with pin monitoring and manual stop.
   */
  VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << RHD5950_CLK_TIM_NUM);
   
  RHD5950_CLK_TIM->ENABLE = 0;

  RHD5950_CLK_TIM->RST_VALUE = (SYSCLK / m_ui32ClockHz) - 1;
  RHD5950_CLK_TIM->PWMA_VALUE = ((SYSCLK / m_ui32ClockHz) - 1) / 2;

  RHD5950_CLK_TIM->CTRL = ((3UL << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos) |
                                  (TIM_PERIPHERAL_CTRL_STATUS_INV_Msk));		
}

/*******************************************************************************
 **
 ** @brief  Enables the ADC.
 **
 ** Sets the enable pin on the ADC, and performs a conversion to put the ADC's
 ** digital logic into a known state.
 **
 ******************************************************************************/
void RHD5950_WakeUp(void)
{   
  /* Set Enable pin. */
  RHD5950_EN_PORT->SETOUT = (1UL << RHD5950_EN_PIN);

  /* Configure Timer for continuous mode. */
  RHD5950_CLK_TIM->CTRL &= ~TIM_PERIPHERAL_CTRL_AUTO_DISABLE_Msk;
  while((RHD5950_BUSY_L_PORT->DATAIN & (1UL << RHD5950_BUSY_L_PIN)) == 0)
  {
    /* Send first clock cycle, */
    RHD5950_CLK_TIM->ENABLE = 1;
  }
  /* Automatically disable the Timer once the last cycle has completed. */
  RHD5950_CLK_TIM->CTRL |= TIM_PERIPHERAL_CTRL_AUTO_DISABLE_Msk;


  /* Perform a conversion to initialize the ADC's internal digital logic. */
  /* Discard return val, it is not a real conversion                      */
  RHD5950_setadcch_and_Sample(0);
}

/*******************************************************************************
 **
 ** @brief  Disables the ADC.
 **
 ******************************************************************************/
void RHD5950_Sleep(void)
{   
	RHD5950_EN_PORT->CLROUT = (1UL << RHD5950_EN_PIN);
}

/*******************************************************************************
 **
 ** @brief  Sets the ADC ch select pins. 
 **
 ** @param  adcch is the ADC channel to sample from. Valid range 0 to 15
 **
 ******************************************************************************/
void RHD5950_setADCch(uint32_t adcch)
{
#ifdef SIP_EVK_REVA_FIX
  // Shifts ADC 1-6 up by one, due to a board layout issue
  // Undef SIP_EVK_REVA_FIX or remove this section if not using EVK REV_A
  // DO NOT connect an input voltage to AN0 on the reva brd - it is tied to AGND on the SIP
  if(adcch == 0) {adcch = 6;}
  else{
   if((adcch > 0) && (adcch < 7)) {adcch--;}
  }
#endif
  
  if(adcch >= RHD5950_NUM_CHANNELS) return;
	uint32_t portword=(adcch & 0xf) << RHD5950_MUX0_PIN;
  RHD5950_MUX0_BANK.DATAOUT=(RHD5950_MUX0_BANK.DATAOUT & ~(0xf<<RHD5950_MUX0_PIN))|(portword);
}
	

/*******************************************************************************
 **
 ** @brief  Performs an ADC conversion on a user-provided ADC channel.
 **
 ** @param  none
 **
 ** @return Returns the 14-bit ADC conversion from the requested channel.
 **
 ******************************************************************************/
uint32_t RHD5950_Sample(void)
{
  uint8_t i;
  uint32_t ui32TempResult, ui32Result;
  volatile uint32_t readChange;
  //volatile uint32_t timeout=1000000;

  /* Pulse STCNV to start conversion, waiting 20nS before starting SCLK. */
  RHD5950_STCNV_PORT->SETOUT = (1UL << RHD5950_STCNV_PIN);

  /* Configure Timer for continuous mode. */
  RHD5950_CLK_TIM->CTRL &= ~TIM_PERIPHERAL_CTRL_AUTO_DISABLE_Msk;

  readChange=RHD5950_EOC_PORT->EDGE_STATUS;

  /* Send first clock cycle, */
  RHD5950_CLK_TIM->ENABLE = 1;

  /* ...and wait for pin to output low. */
  while((RHD5950_CLK_PORT->DATAIN & (1UL << RHD5950_CLK_PIN)) != 0)
  {
  }

  /* Disable STCNV signal. */
  RHD5950_CLK_PORT->CLROUT = (1UL << RHD5950_STCNV_PIN);  

  /* ...and wait for pin to output high. */
  while((RHD5950_CLK_PORT->DATAIN & (1UL << RHD5950_CLK_PIN)) == 0)
  {
  }

  /* Wait for Conversion to strobe EOC_H */
  //Warning: any ISR read of EDGE_STATUS will auto-clear this reg 
  //and code may miss EOC strobe
  //while((RHD5950_EOC_PORT->EDGE_STATUS & (1UL<<RHD5950_EOC_PIN) )==0)
  while((RHD5950_BUSY_L_PORT->DATAIN & (1UL << RHD5950_BUSY_L_PIN)) == 0)
  {
  }

  /* Automatically disable the Timer once the last cycle has completed. */
  RHD5950_CLK_TIM->CTRL |= TIM_PERIPHERAL_CTRL_AUTO_DISABLE_Msk;

  /* Read result placed on parallel bus. */
  ui32TempResult = (RHD5950_DATA0_PORT->DATAIN & (0x3fff<<RHD5950_DATA0_POS)) >> RHD5950_DATA0_POS;

  /* Flip the bits around (bit0->bit13, bit13->bit0) */
  ui32Result = 0;
  for(i=0; i<RHD5950_DATA_WIDTH; i++)
  {
    ui32Result |= (ui32TempResult & (0x1UL)) << (RHD5950_DATA_WIDTH-1-i);
    ui32TempResult >>= 1;
  }

  return(ui32Result);
}

/*******************************************************************************
 **
 ** @brief  Performs an ADC conversion on a user-provided ADC channel.
 **
 ** @param  adcch - the ADC channel (0-15)
 **
 ** @return Returns the 14-bit ADC conversion from the requested channel.
 **
 ******************************************************************************/
uint32_t RHD5950_setadcch_and_Sample(uint32_t adcch){
	RHD5950_setADCch(adcch);
	return RHD5950_Sample();
}

/*******************************************************************************
 **
 ** @brief  Converts an ADC sample to a four digit voltage.
 **
 ** @param  ui32Sample is the ADC sample with a maximum value of 0x3FFF.
 **
 ** @return Returns the decimal value of the converted voltage in mV, with a 
 **         maximum value of 4999.
 **
 ******************************************************************************/
uvolt1000_t RHD5950_SampleToVoltage1000(uint32_t ui32Sample)
{
	//11 is a manual precalc'ed value to maximize accuracy without overflow
	ui32Sample*=(uint32_t)((1UL<<11)*1000*RHD5950_VREF/0x3fff);
  return (ui32Sample>>11);
}
 
/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
